#include <stdio.h>
#include <time.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <string.h>

extern "C" {
#include "error.h"
}
#include "pl.h"
#include "utils.h"

char *state_str[4] = { " OK ", "WARN", "CRIT", " ?? " };

void version(void)
{
	printf("check_nagios_summary, v" VERSION " (C) 2007 by folkert@vanheusden.com\n");
}

void usage(void)
{
	version();

	printf("\n");

	printf("-f     path to status.log ('status_file' parameter in nagios.cfg)\n");
	printf("-F     host:port for retrieving status.log\n");
	printf("-x     status.log is in nagios 1.0 format\n");
	printf("-X     status.log is in nagios 2.0 format\n");
	printf("-h     show only state type 'HARD' (default)\n");
	printf("-H     show also state type 'SOFT'\n");
	printf("-V     show version & exit\n");
}

int main(int argc, char *argv[])
{
	int c;
	char *statuslog = "/usr/local/nagios/var/status.log";
	char is_20_format = 1;
	char is_file = 1;
	char hard_only = 1;
	struct stats *cur = NULL;
	int n_cur = 0;
	int loop;
	int fd;
	int selected_state = -1;
	char *str_out = NULL;
	int str_out_len = 0;

	while((c = getopt(argc, argv, "xXF:hHf:i:V")) != -1)
	{
		switch(c)
		{
			case 'h':
				hard_only = 1;
				break;

			case 'H':
				hard_only = 0;
				break;

			case 'x':
				is_20_format = 0;
				break;

			case 'X':
				is_20_format = 1;
				break;

			case 'f':
				statuslog = optarg;
				is_file = 1;
				break;

			case 'F':
				statuslog = optarg;
				is_file = 0;
				break;

			case 'V':
				version();
				return 0;

			default:
				usage();
				return 0;
		}
	}


	if (is_file == 1)     /* file */
		fd = open64(statuslog, O_RDONLY);
	else
		fd = connect_to(statuslog);
	if (fd == -1)
	{
		printf("WARNING cannot obtain nagios status\n");
		exit(1);
	}

	if (is_20_format)
		parse_2_0_statuslog(fd, &cur, &n_cur);
	else
		parse_1_0_statuslog(fd, &cur, &n_cur);

	close(fd);

	for(loop=0; loop<n_cur; loop++)
	{
		if (should_i_show_entry(cur, n_cur, loop, 0, 0, 0, 1))
		{
			char buffer[128];
			int buffer_len;

			selected_state = max(selected_state, cur[loop].current_state);

			buffer_len = snprintf(buffer, sizeof(buffer), "%s/%s: %s, ", cur[loop].host_name?cur[loop].host_name:"", cur[loop].service_description?cur[loop].service_description:"", state_str[cur[loop].current_state]);

			str_out = (char *)realloc(str_out, str_out_len + buffer_len + 1);

			memcpy(&str_out[str_out_len], buffer, buffer_len + 1);
			str_out_len += buffer_len;
		}
	}

	if (selected_state == -1)
	{
		selected_state = 0;
	}

	if (str_out)
	{
		printf("%s %s\n", state_str[selected_state], str_out);
	}
	else
	{
		printf("OK all fine");
	}

	return selected_state;
}
